// ponyfill for requestIdleCallback

import { now } from './timing.js';

function deadline(endTime) {
  return {
    timeRemaining() {
      return Math.max(endTime - now(), 0);
    },
    didTimeout: false,
  };
}

const _requestIdleCallback =
  typeof requestIdleCallback !== 'undefined' ?
  requestIdleCallback :
  typeof requestAnimationFrame !== 'undefined' ?
  function(callback) {
    return requestAnimationFrame(startTime => {
      callback(deadline(startTime + 16));
    });
  } :
  function(callback) {
    return setTimeout(() => {
      callback(deadline(now() + 16));
    }, 20);
  };

const _cancelIdleCallback =
  typeof requestIdleCallback !== 'undefined' ?
  cancelIdleCallback :
  typeof requestAnimationFrame !== 'undefined' ?
  cancelAnimationFrame :
  clearTimeout;

export { _requestIdleCallback as requestIdleCallback, _cancelIdleCallback as cancelIdleCallback };
