import sameOrigin from './input/same-origin.js';
import crossOrigin from './input/cross-origin-with-filter.js';
import element from './input/element.js';
import preview from './output/preview.js';
import * as Pattern from './pattern.js';

// just GUI stuff from here on. horrible, horrible GUI stuff.

let runningTask, canvas, victim;
const dimensions = { x1: 0, y1: 0, x2: 0, y2: 0 };

function stop() {
  if(runningTask) {
    runningTask.cancel();
    runningTask = undefined;
  }
  if(canvas && canvas.parentNode) {
    canvas.parentNode.removeChild(canvas);
  }
  document.getElementById('initial-parameters').style.display = 'block';
  document.getElementById('secondary-parameters').style.display = 'none';
  document.getElementById('in-progress').style.display = 'none';
}

function next() {
  const inputType = document.getElementById('input-type').value;
  if(inputType === 'element') {
    document.getElementById('html-input-secondary-parameters').style.display = 'block';
    document.getElementById('arena-width').value = 1024;
    document.getElementById('arena-height').value = 1024;
    
    victim = document.createElement('div');
    victim.style.backgroundColor = 'white';
    const innerVictim = document.createElement('div');
    innerVictim.innerHTML = document.getElementById('html-content').value;
    victim.appendChild(innerVictim);
  } else {
    document.getElementById('html-input-secondary-parameters').style.display = 'none';
    document.getElementById('arena-width').value = 48;
    document.getElementById('arena-height').value = 48;
    
    victim = document.createElement('img');
    victim.src = document.getElementById('image-url').value;
  }
  
  dimensions.x1 = dimensions.x2 = dimensions.y1 = dimensions.y2 = -1000;
  updateSelectionArea();
  
  if(inputType === 'same-origin') {
    document.getElementById('arena-secondary-parameters').style.display = 'none';
  } else {
    document.getElementById('arena-secondary-parameters').style.display = 'block';
  }
  
  document.getElementById('victim-container').appendChild(victim);
  
  document.getElementById('initial-parameters').style.display = 'none';
  document.getElementById('secondary-parameters').style.display = 'block';
  document.getElementById('in-progress').style.display = 'none';
}

function start() {
  const inputType = document.getElementById('input-type').value;
  
  const rectangle = {
    x: Math.min(dimensions.x1, dimensions.x2),
    y: Math.min(dimensions.y1, dimensions.y2),
    width: Math.abs(dimensions.x1 - dimensions.x2) + 1,
    height: Math.abs(dimensions.y1 - dimensions.y2) + 1
  };
  if(rectangle.x < 0 && rectangle.y < 0) {
    if(inputType === 'element') {
      return;
    } else {
      rectangle.x = rectangle.y = 0;
      rectangle.width = victim.width;
      rectangle.height = victim.height;
    }
  }
  
  canvas = document.createElement('canvas');
  canvas.id = 'preview';
  const baseOutput = preview(canvas);
  const output = {
    reset() {
      return baseOutput.reset.apply(baseOutput, arguments);
    },
    nextZ() {
      return baseOutput.nextZ.apply(baseOutput, arguments);
    },
    put() {
      document.getElementById('unstarted-text').style.display = 'none';
      document.getElementById('started-text').style.display = 'block';
      document.getElementById('finished-text').style.display = 'none';
      
      output.put = function put() {
        return baseOutput.put.apply(baseOutput, arguments);
      };
      
      return baseOutput.put.apply(baseOutput, arguments);
    },
    finish() {
      document.getElementById('unstarted-text').style.display = 'none';
      document.getElementById('started-text').style.display = 'none';
      document.getElementById('finished-text').style.display = 'block';
      
      return baseOutput.finish.apply(baseOutput, arguments);
    },
  };
  output.reset(rectangle.width, rectangle.height);
  document.getElementById('in-progress').appendChild(canvas);
  
  if(victim.parentNode) {
    victim.parentNode.removeChild(victim);
  }
  
  const pattern = Pattern.offset(
    Pattern.progressive(
      rectangle.width,
      rectangle.height,
      document.getElementById('bit-depth').value |0,
      output,
      {
        color: document.getElementById('use-color').checked
      }
    ),
    rectangle.x,
    rectangle.y
  );
  
  if(inputType === 'element') {
    const whiteSheen = document.createElement('div');
    whiteSheen.id = 'white-sheen';
    whiteSheen.style.left = `${rectangle.x}px`;
    whiteSheen.style.top = `${rectangle.y}px`;
    whiteSheen.style.width = `${rectangle.width}px`;
    whiteSheen.style.height = `${rectangle.height >> 1}px`;
    victim.appendChild(whiteSheen);
    const blackSheen = document.createElement('div');
    blackSheen.id = 'black-sheen';
    blackSheen.style.left = `${rectangle.x}px`;
    blackSheen.style.top = `${rectangle.y + (rectangle.height >> 1)}px`;
    blackSheen.style.width = `${rectangle.width}px`;
    blackSheen.style.height = `${rectangle.height >> 1}px`;
    victim.appendChild(blackSheen);
    
    runningTask = element(
      victim,
      {
        set(color) {
          if(color === 'white') {
            return {
              x: rectangle.x + Math.random() * rectangle.width,
              y: rectangle.y + Math.random() * rectangle.height * 0.5,
              value: Math.random() * 0.9,
            };
          } else if(color === 'black') {
            return {
              x: rectangle.x + Math.random() * rectangle.width,
              y: rectangle.y + (Math.random() + 1) * rectangle.height * 0.5,
              value: 0.1 + Math.random() * 0.9,
            };
          }
        },
        finish() {
          whiteSheen.style.backgroundColor = blackSheen.style.backgroundColor = 'rgba(128,128,128,0.01)';
        },
      },
      pattern,
      {
        arenaWidth: document.getElementById('arena-width').value |0,
        arenaHeight: document.getElementById('arena-height').value |0,
        calibrationSteps: document.getElementById('calibration-steps').value |0,
        waitTimeForLoad: (+document.getElementById('wait-time-for-load').value * 1000) |0,
        useSoftwareify: document.getElementById('use-softwareify').checked,
        useDoubleFrame: document.getElementById('use-double-frame').checked,
        layerCount: document.getElementById('layer-count').value |0
      }
    );
  } else if(inputType === 'cross-origin-with-filter') {
    runningTask = crossOrigin(
      victim,
      pattern,
      {
        arenaWidth: document.getElementById('arena-width').value |0,
        arenaHeight: document.getElementById('arena-height').value |0,
        calibrationSteps: document.getElementById('calibration-steps').value |0
      }
    );
  } else if(inputType === 'same-origin') {
    runningTask = sameOrigin(
      victim,
      pattern
    );
  } else {
    alert(`"${inputType}" is not a valid input type!`);
    return;
  }
  
  document.getElementById('unstarted-text').style.display = 'block';
  document.getElementById('started-text').style.display = 'none';
  document.getElementById('finished-text').style.display = 'none';
  
  document.getElementById('initial-parameters').style.display = 'none';
  document.getElementById('secondary-parameters').style.display = 'none';
  document.getElementById('in-progress').style.display = 'block';
}

document.getElementById('stop-button').addEventListener('click', stop);
document.getElementById('next-button').addEventListener('click', next);
document.getElementById('start-button').addEventListener('click', start);

document.getElementById('use-softwareify').checked = navigator.userAgent.indexOf('Chrome/') !== -1;
document.getElementById('use-double-frame').checked = navigator.userAgent.indexOf('Chrome/') !== -1;

document.getElementById('input-type').addEventListener('change', () => {
  const value = document.getElementById('input-type').value;
  if(value === 'element') {
    document.getElementById('html-input-parameters').style.display = 'block';
    document.getElementById('image-input-parameters').style.display = 'none';
  } else {
    document.getElementById('html-input-parameters').style.display = 'none';
    document.getElementById('image-input-parameters').style.display = 'block';
  }
});

function updateSelectionArea() {
  const selectionArea = document.getElementById('victim-selection-area');
  selectionArea.style.display = 'block';
  selectionArea.style.left = `${Math.min(dimensions.x1, dimensions.x2) - 1}px`;
  selectionArea.style.top = `${Math.min(dimensions.y1, dimensions.y2) - 1}px`;
  selectionArea.style.width = `${Math.abs(dimensions.x1 - dimensions.x2) + 1}px`;
  selectionArea.style.height = `${Math.abs(dimensions.y1 - dimensions.y2) + 1}px`;
}

let isSelecting = false;

document.getElementById('victim-container').addEventListener('mousedown', event => {
  dimensions.x1 = dimensions.x2 = event.offsetX |0;
  dimensions.y1 = dimensions.y2 = event.offsetY |0;
  updateSelectionArea();
  isSelecting = true;
  event.preventDefault();
});

document.getElementById('victim-container').addEventListener('mousemove', event => {
  if(isSelecting) {
    dimensions.x2 = event.offsetX |0;
    dimensions.y2 = event.offsetY |0;
    updateSelectionArea();
    event.preventDefault();
  }
});

window.addEventListener('mouseup', () => {
  isSelecting = false;
});

stop();
