// this bit of code handles the progressive pattern. it's really weird and not
// particularly important, so I'm not going to try to explain it.

function nextPowerOf2(v) {
  --v;
  v |= v >> 1;
  v |= v >> 2;
  v |= v >> 4;
  v |= v >> 8;
  v |= v >> 16;
  return v + 1;
}

function* progressive1D(length) {
  if(length <= 0) {
    return;
  }
  
  yield function*() {
    yield 0;
  };
  
  for(let stride = nextPowerOf2(length); stride > 1; stride = stride >> 1) {
    yield function*() {
      for(let x = stride >> 1; x < length; x += stride) {
        yield x;
      }
    };
  }
}

export function* progressive(width, height, depth, output, options) {
  const channels = options && options.color ? ['r', 'g', 'b'] : ['brightness'];
  
  const squareLength = Math.max(width, height);
  let blockSize = nextPowerOf2(squareLength);
  const visited = [];
  
  for(const spots of progressive1D(squareLength)) {
    for(const y of spots()) {
      if(y >= height) continue;
      for(const spots of visited) {
        for(const x of spots()) {
          if(x >= width) continue;
          for(const channel of channels) {
            for(let z = 0; z < depth; ++z) {
              const xyz = { x, y, channel };
              xyz.z = output.nextZ(xyz);
              output.put(xyz, yield xyz, blockSize << 1, blockSize);
            }
          }
        }
      }
    }
    
    visited.push(spots);
    
    for(const x of spots()) {
      if(x >= width) continue;
      for(const spots of visited) {
        for(const y of spots()) {
          if(y >= height) continue;
          for(const channel of channels) {
            for(let z = 0; z < depth; ++z) {
              const xyz = { x, y, channel };
              xyz.z = output.nextZ(xyz);
              output.put(xyz, yield xyz, blockSize, blockSize);
            }
          }
        }
      }
    }
    
    blockSize >>= 1;
  }
  
  output.finish();
}

// offset the points of a given pattern by (x, y). it will still feed its data
// back to the output at the original positions.
export function* offset(pattern, x, y) {
  for(let next = pattern.next(); !next.done;) {
    const to = {};
    for(const key in next.value) {
      to[key] = next.value[key];
    }
    to.x += x;
    to.y += y;
    next = pattern.next(yield to);
  }
}
